@echo off
rem ***********************************************************
rem Create preprocessed administrative boundaries
rem   parameters: <country> <mkgmap-release>
rem Example: CreateBoundaries Europe 1700
rem ***********************************************************

echo **********************************************************
echo %0 %1 %2
echo **********************************************************
SETLOCAL

if [%1]==[] goto tagEndParmError
if [%2]==[] goto tagEndParmError

set COUNTRY=%1
set MKGMAP_REL=%2
set SOURCE=..\PlanetFiles\%COUNTRY%.osm.pbf
set BOUNDS_DIR=..\Boundaries
set OUTPUT_DIR=%BOUNDS_DIR%\%COUNTRY%
set O5M_FILE=%OUTPUT_DIR%.o5m
set O5M_FILTERED_FILE=%OUTPUT_DIR%.filtered.o5m

if not "%3"=="" set SOURCE=%3 

if not exist "%BOUNDS_DIR%" mkdir "%BOUNDS_DIR%"
if exist "%OUTPUT_DIR%" goto tagClear
	mkdir "%OUTPUT_DIR%"
	goto tagDirIsEmtpy
:tagClear
del "%OUTPUT_DIR%\"
:tagDirIsEmtpy

if defined ProgramFiles(x86) (
	echo 64bit
	set JVM_MEMORY=8G
) else (
	set JVM_MEMORY=1400M
)

echo Convert planet file to o5m
mkgmap\osmconvert.exe "%SOURCE%" -o="%O5M_FILE%"

echo Filter boundaries
mkgmap\osmfilter.exe "%O5M_FILE%"^
	--keep-nodes=^
    --keep-ways-relations="boundary=administrative =postal_code postal_code="^
    --out-o5m > "%O5M_FILTERED_FILE%"

echo Extract boundaries
java -Dlog.config=config\mkgmap.logging.properties -Xmx%JVM_MEMORY%^
	-cp mkgmap\mkgmap-r%MKGMAP_REL%\mkgmap.jar uk.me.parabola.mkgmap.reader.osm.boundary.BoundaryPreprocessor^
	"%O5M_FILTERED_FILE%"^
	"%OUTPUT_DIR%"

del "%O5M_FILE%"
del "%O5M_FILTERED_FILE%"	
	
exit /b 0

:tagEndParmError
echo Missing parameters
exit /b 1
