@echo off
rem ***********************************************************
rem WARNING: Nuvi maps must not exceed 2 GB size limit!!
rem ***********************************************************

rem ***********************************************************
rem <country> <mkgmap-release> [<source>]
rem Example1: Create GermanyPlus 1700
rem Example2: Create GermanyPlus 1700 ..\TilesGermanyPlus\*.gz
rem ***********************************************************

rem ***********************************************************
rem Procedure in order to create partial European maps
rem
rem 1. Get tile list using FancyRoute (menu-Tools-Garmin Map Analysis-Commands-List Boundary Box Tiles
rem 2. Create new tiles folder, name it e.g. "EasternEurope"
rem 3. Copy tiles from 1. to the folder created in 2.
rem 4. Call CreateNuviMap EasternEurope 3498
rem 5. Check if map size is < 2MB
rem ***********************************************************

SETLOCAL

echo **********************************************************
echo %0 %1
echo **********************************************************

if [%1]==[] goto tagEndParmError
if [%2]==[] goto tagEndParmError

rem retrieve date
for /F "tokens=2 delims==." %%I in ('%SystemRoot%\System32\wbem\wmic.exe OS GET LOCALDATETime /VALUE') do set "LOCALDATE=%%I"
set "LOCALDATE=%LOCALDATE:~0,4%-%LOCALDATE:~4,2%-%LOCALDATE:~6,2%"

set COUNTRY=%1
set MKGMAP_REL=%2
set SOURCE=..\Tiles\%COUNTRY%\*.pbf
set MAP_DIR=..\Maps\%COUNTRY%Nuvi

if not "%3"=="" set SOURCE=%3 

if exist "%MAP_DIR%" goto tagClear
	mkdir "%MAP_DIR%"
	goto tagDirIsEmtpy
:tagClear
del "%MAP_DIR%\"
:tagDirIsEmtpy

if defined ProgramFiles(x86) (
	echo 64bit
	set JVM_MEMORY=8G
) else (
	set JVM_MEMORY=1400M
)

java -jar -Dlog.config=config\mkgmap.logging.properties -Xmx%JVM_MEMORY% mkgmap\mkgmap-r%MKGMAP_REL%\mkgmap.jar^
	--style-file=.\style^
	--list-styles^
	--check-styles^
	--style=snNuvi^
	--preserve-element-order^
	--latin1^
	--gmapsupp^
	--add-pois-to-areas^
	--remove-short-arcs^
	--merge-lines^
	--route^
	--index^
	--bounds="..\Boundaries\Europe"^
	--x-split-name-index^
	--process-exits^
	--housenumbers^
	--reduce-point-density=4^
	--reduce-point-density-polygon=8^
	--precomp-sea=..\PlanetFiles\sea-latest.zip^
	--remove-ovm-work-files^
	--max-jobs=4^
	--family-name="OSM %COUNTRY%"^
	--description="OpenStreetMap %COUNTRY%(routable) %LOCALDATE%"^
	--output-dir="%MAP_DIR%"^
	%SOURCE% %4 %5 %6 %7 %8 %9 

:tagCleanUp
echo Cleanup img files
for %%i in ("%MAP_DIR%\*") do if not %%~nxi == gmapsupp.img del %%i
	
exit /b 0

:tagEndParmError
echo Missing parameters
exit /b 1
